searchNodes=[{"doc":"Glock is a simple websocket client application based on the Gun HTTP/HTTP2/Websocket Erlang library. Glock aims to simplify the specific task of starting and configuring a websocket client connection to a remote server, providing common default values for all connection settings provided by Gun while still allowing for full customization. Glock also provides a set of callbacks for processing messages sent to and received from the remote server and tracking state across the life of the connection in whatever way makes sense to your application. Default callback implementations let you get up and running immediately for simply sending messages to a server and logging received responses by implementing the __using__/1 macro: Example: defmodule MySocket do use Glock.Socket end iex&gt; {:ok, conn} = MySocket.start_link(host: &quot;localhost&quot;, path: &quot;/ws&quot;) iex&gt; :ok = MySocket.send(conn, &quot;hello socket!&quot;) :ok Implementing the init_stream/1 callback allows you to create and store state for the socket connection which can be accessed from subsequent message send or receive events. A simple example might be to count the number of messages sent and received from the socket. Example: defmodule MySocket do use Glock.Socket def init_stream(conn: conn, protocols: _, headers: _) do %{ &quot;connection&quot; =&gt; conn.stream, &quot;sent&quot; =&gt; 0, &quot;received&quot; =&gt; 0 } end end Implementing the handle_send/2 callback allows for customization of the message frame types and the encoding or serialization performed on messages prior to sending. Piggy-backing the prior example, a complex data structure could be serialized to JSON and counted before being sent to the remote server. Example: defmodule MySocket do ... def handle_send(message, state) do frame = {:text, JSON.encode(message)} new_state = Map.put(state, &quot;sent&quot;, state[&quot;sent&quot;] + 1) {frame, new_state} end end Finally, implementing the handle_receive/2 callback allows for custom handling of messages beyond simply logging them. All messages received by the gun connection pass through the handle_receive/2 callback, so you can decode them, store them, reprocess them or anything else you like. This handler also covers receiving :close control frames and cleaning up/ shutting down the glock process appropriately according to the Websocket specification. Example: defmodule MySocket do ... def handle_receive(frame, state) do case frame do {:text, message} -&gt; send_to_internal_queue(message) new_state = Map.put(state, &quot;received&quot;, state[&quot;received&quot;] + 1) {:doesnt_matter, {:ok, new_state}} :close -&gt; JSON.encode(state) |&gt; write_to_log_service() {:close, {:close, state}} _ -&gt; {:doesnt_matter, {:ok, state}} end end end","ref":"Glock.html","title":"Glock","type":"behaviour"},{"doc":"Processes messages sent from the client application to the websocket server and optionally tracks state for the connection. Messages sent by either the send/2 or send_async/2 functions are passed through handle_send/2 for processing. The c:handle_send/2 callback for the module implementing the glock behaviour is responsible for packaging messages to be sent into an appropriate websocket message frame. Terms must be converted or serialized to an appropriate text (string) or raw binary encoding and wrapped in a tuple indicating their format, or else a :close control frame can be sent.","ref":"Glock.html#c:handle_push/2","title":"Glock.handle_push/2","type":"callback"},{"doc":"Processes messages received by the client application from the remote websocket server. Any application-specific operations that must be done on received messages are performed by the handle_receive/2 callback. Based on the frame received and the optionally tracked state available to the callback, emits a response frame and triggers the appropriate action. If an {:ok, state} tuple is produced, there is no expectation that a frame should be sent back to the websocket server. If a {:send, state} tuple is produced, the response frame is sent to the server. If a {:close, state} tuple is produced, a simple :close frame is returned to the server to signal the client&#39;s acknowledgement the connection is to be terminated and the process exits, cleaning up its state.","ref":"Glock.html#c:handle_receive/2","title":"Glock.handle_receive/2","type":"callback"},{"doc":"Initialize the state of the socket implementing the glock behaviour. The state is stored within the glock connection and passed to the handler callbacks when messages are sent or received over the socket.","ref":"Glock.html#c:init_stream/1","title":"Glock.init_stream/1","type":"callback"},{"doc":"","ref":"Glock.html#is_close/1","title":"Glock.is_close/1","type":"macro"},{"doc":"See Glock.Stream.stream/1.","ref":"Glock.html#stream/1","title":"Glock.stream/1","type":"function"},{"doc":"The types of frame structures accepted and returned by underlying gun client for handling.","ref":"Glock.html#t:frame/0","title":"Glock.frame/0","type":"type"},{"doc":"Keyword list of arguments to initialize a glock socket process. All keys are optional with default values except for :host and :path.","ref":"Glock.html#t:init_opts/0","title":"Glock.init_opts/0","type":"type"},{"doc":"Options available to the init_stream/1 callback for initializing the state of socket implementing the glock behaviour.","ref":"Glock.html#t:stream_init_opts/0","title":"Glock.stream_init_opts/0","type":"type"},{"doc":"Defines the glock connection struct that serves as the configuration state of an initialized glock process. The struct tracks all configuration settings and arguments passed into the connection when it is initialized and provides common default values for all settings except for the host and path of the remote websocket server. Provides utility functions for creating and ensuring the proper default values are set within the connection struct.","ref":"Glock.Conn.html","title":"Glock.Conn","type":"module"},{"doc":"Reduces over a keyword list of arguments for configuring the glock process and adds them to an empty instance of the Glock.Conn.t struct. Configs are merged with values passed by the user superseding default values with the exception of the http protocol which is locked to HTTP/1.1 for websocket compatibility.","ref":"Glock.Conn.html#new/1","title":"Glock.Conn.new/1","type":"function"},{"doc":"","ref":"Glock.Conn.html#t:t/0","title":"Glock.Conn.t/0","type":"type"},{"doc":"Defines the genserver that starts and manages the underlying gun process. The glock socket is configured to trap exits and monitor the gun connection to the remote server. In the event of failure, glock establishes a new http connection to the server if possible immediately attempts to re-upgrade to a websocket connection. The __using__/1 macro provides default implementations of the three glock callback functions: c:init_stream/1, c:handle_send/2, and c:handle_receive/2, and marks them as overridable to allow for customization. Provides functions for sending messages to the glock process for handling and relaying to the remote websocket server both synchronously and asynchronously.","ref":"Glock.Socket.html","title":"Glock.Socket","type":"module"},{"doc":"Implements a Glock websocket connection as an Elixir Stream resource, returning messages as they are received until the connection is terminated, either by the remote websocket server or by exiting the Glock process.","ref":"Glock.Stream.html","title":"Glock.Stream","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"Glock.Stream.html#child_spec/1","title":"Glock.Stream.child_spec/1","type":"function"},{"doc":"Synchronously send a message to the remote server via the glock process.","ref":"Glock.Stream.html#push/2","title":"Glock.Stream.push/2","type":"function"},{"doc":"Asynchronously send a message to the remote server via the glock process.","ref":"Glock.Stream.html#push_async/2","title":"Glock.Stream.push_async/2","type":"function"},{"doc":"Start a named glock process outside of a supervision tree, passing all http and websocket configuration options for initialization.","ref":"Glock.Stream.html#start/1","title":"Glock.Stream.start/1","type":"function"},{"doc":"Start a named glock process and link it to the calling process, passing all http and websocket configuration options for initialization.","ref":"Glock.Stream.html#start_link/1","title":"Glock.Stream.start_link/1","type":"function"},{"doc":"","ref":"Glock.Stream.html#stream/1","title":"Glock.Stream.stream/1","type":"function"},{"doc":"","ref":"Glock.ConnError.html","title":"Glock.ConnError","type":"exception"},{"doc":"Glock Glock is a simple websocket client application based on the Gun HTTP/HTTP2/Websocket Erlang library. Glock aims to simplify the specific task of starting and configuring a websocket client connection to a remote server, providing common default values for all connection settings provided by Gun while still allowing for full customization. Glock also provides a set of callbacks for processing messages sent to and received from the remote server and tracking state across the life of the connection in whatever way makes sense to your application. Default callback implementations let you get up and running immediately for simply sending messages to a server and logging received responses by implementing the __using__/1 macro: Example: defmodule MySocket do use Glock.Socket end iex&gt; {:ok, conn} = MySocket.start_link(host: &quot;localhost&quot;, path: &quot;/ws&quot;) {:ok, #PID&lt;0.260.0&gt;} iex&gt; :ok = MySocket.send(conn, &quot;hello socket!&quot;) Implementing the init_stream/1 callback allows you to create and store state for the socket connection which can be accessed from subsequent message send or receive events. A simple example might be to count the number of messages sent and received from the socket. Example: defmodule MySocket do use Glock.Socket def init_stream(conn: conn, protocols: _, headers: _) do %{ &quot;connection&quot; =&gt; conn.stream, &quot;sent&quot; =&gt; 0, &quot;received&quot; =&gt; 0 } end end Implementing the handle_send/2 callback allows for customization of the message frame types and the encoding or serialization performed on messages prior to sending. Piggy-backing the prior example, a complex data structure could be serialized to JSON and counted before being sent to the remote server. Example: defmodule MySocket do ... def handle_send(message, state) do frame = {:text, JSON.encode(message)} new_state = Map.put(state, &quot;sent&quot;, state[&quot;sent&quot;] + 1) {frame, new_state} end end Finally, implementing the handle_receive/2 callback allows for custom handling of messages beyond simply logging them. All messages received by the gun connection pass through the handle_receive/2 callback, so you can decode them, store them, reprocess them or anything else you like. This handler also covers receiving :close control frames and cleaning up/ shutting down the glock process appropriately according to the Websocket specification. Example: defmodule MySocket do ... def handle_receive(frame, state) do case frame do {:text, message} -&gt; send_to_internal_queue(message) new_state = Map.put(state, &quot;received&quot;, state[&quot;received&quot;] + 1) {:doesnt_matter, {:ok, new_state}} :close -&gt; JSON.encode(state) |&gt; write_to_log_service() {:close, {:close, state}} _ -&gt; {:doesnt_matter, {:ok, state}} end end end","ref":"readme.html","title":"Glock","type":"extras"},{"doc":"The package can be installed by adding glock to your list of dependencies in mix.exs: def deps do [ {:glock, &quot;~&gt; 0.1.0&quot;} ] end The docs can be found at https://hexdocs.pm/glock.","ref":"readme.html#installation","title":"Glock - Installation","type":"extras"}]